// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/OPTO32/OPTO32_Linux_7.x.x.x_GSC_DN/driver/main.h $
// $Rev: 53212 $
// $Date: 2023-06-20 08:57:52 -0500 (Tue, 20 Jun 2023) $

// OPTO32: Device Driver: header file

#ifndef	__MAIN_H__
#define	__MAIN_H__

#include "opto32.h"

#define	DEV_BAR_SHOW						0
#define	DEV_METRICS_SHOW					0
#define	DEV_PCI_ID_SHOW						0
#define	DEV_SUPPORTS_IRQ
#define	DEV_SUPPORTS_WAIT
#define	GSC_ALT_DEV_GET(a)					(a)
#define	GSC_ALT_STRUCT_GET(i,d)				(d)
#define	GSC_ALT_STRUCT_T					dev_data_t
#define	GSC_DEVS_PER_BOARD					1
typedef struct _dev_data_t					dev_data_t;

#include "gsc_main.h"



// macros *********************************************************************

#define	DEV_MODEL							"OPTO32"	// Upper case form of the below.
#define	DEV_NAME							"opto32"	// MUST AGREE WITH OPTO32_BASE_NAME
#define	DEV_NAME_LC							DEV_NAME
#define	DEV_NAME_UC							DEV_MODEL

#define	DEV_VERSION_STR						GSC_STR(DEV_VERSION_MAJ_NUM) "." GSC_STR(DEV_VERSION_MIN_NUM)
#define	DEV_VERSION							DEV_VERSION_STR

#define	DEV_VERSION_MAJ_NUM					8
#define	DEV_VERSION_MIN_NUM					7
// 8.7	Updated to support the 6.x series kernel.
//		Updated comments for consistency.
// 8.6	Updated some comments for consistency.
// 8.5	Modified for use under Windows.
//		Removed DMA code as the driver doesn't use DMA.
//		Removed unused macros.
//		White space cleanup.
// 8.4	Modified for use under Windows.
// 8.3	Updated initial file content for consistency.
//		Added support for the 5.x kernel series.
//		Added Endianness support.
// 8.2	Bug fix: was ignoring BAR setup return value.
//		Updated per changes to the OS specific PCI services.
//		Made updates for DEV_SUPPORTS_XXX macro changes.
//		Modified code so open succeeds even if initialization fails.
//		Corrected bug in US and MS debounce IOCTL services.
// 8.1	The BAR code has been updated to include the data under a single structure.
//		The register definitions have been updated.
//		I/O services now pass around an os_mem_t structure instead of a buffer pointer.
//		White space cleanup.
//		Changed the arbitrary wait event callback argument to a void* type.
//		Modified to allow multiple apps to simultaneously access a single device.
//		Correct access to BAR0 and BAR1 (is RO, was RW).
//		Corrected the IOCTL code for the register write and mod services.
// 8.0	Updated to use the newer common driver sources.
//		Removed LINT remarks from sources.
//		Removed GNU notice from non-Linux specific files.
//		Removed Linux specific content from non-Linux specific source files.
//		White space cleanup.
//		Now using a spinlock rather than enabling and disabling interrupts.
//		Updated gsc_irq_open() and gsc_irq_close().
//		Updated gsc_dma_open() and gsc_dma_close().
// 7.7	Reduced #include list in driver interface header.
//		White space cleanup.
// 7.6	Corrected an IOCTL table initialization value.
// 7.5	Linted the sources.
//		Fixed a bug in the OPTO32_IOCTL_DEBOUNCE_US service.
// 7.4	Updated for the 3.x kernel.
// 7.3	BAR0 and BAR2 are now the only BAR regions used.
//		Include all common source, though not all are used.
// 7.2	Eliminated the global dev_check_id() routine.
// 7.1	Modified so BAR2 could be memory or I/O mapped.
// 7.0	Initial release of this driver. This replaces the SDK driver.

// WAIT services
#define	DEV_WAIT_GSC_ALL					OPTO32_WAIT_GSC_ALL
#define	DEV_WAIT_ALT_ALL					OPTO32_WAIT_ALT_ALL
#define	DEV_WAIT_IO_ALL						OPTO32_WAIT_IO_ALL



// data types *****************************************************************

struct _dev_io_t
{
	// Initialized by open and IOCTL services (initialize and/or service specific).

	// Initialized by I/O service at time of use.

	gsc_dma_ch_t*		dma_channel;		// Use this channel for DMA.

	// Initialized by device specific I/O create code.

	u32					io_reg_offset;		// Offset of device's I/O FIFO.

	// Initialize by GSC commone create code.

};

struct _dev_data_t
{
	// Put an integral data type at beginning of structure.
	int					board_index;	// Index of the device being accessed.
	gsc_dev_type_t		board_type;		// Corresponds to basic device type.
	const char*			model;			// Base model number in upper case.
	int					share;			// Were we opened in shared mode?
	int					users;			// Number of currently active open() requests.

	os_pci_t			pci;			// The kernel PCI device descriptor.
	os_data_t			os;				// OS specific data.
	os_spinlock_t		spinlock;		// Control ISR access.
	os_sem_t			sem;			// Control thread access.

	gsc_bar_t			bar;			// device register mappings
	gsc_dma_t			dma;			// For interrupts only.
	gsc_irq_t			irq;			// For interrupts.
	gsc_wait_node_t*	wait_list;

	struct
	{
		// The DMA registers are not used, but required.
		VADDR_T			plx_intcsr_32;		// Interrupt Control/Status Register
		VADDR_T			plx_dmaarb_32;		// DMA Arbitration Register
		VADDR_T			plx_dmathr_32;		// DMA Threshold Register

		VADDR_T			gsc_bcsr_8;		// Board Control/Status Register
		VADDR_T			gsc_cdr_32;		// Clock Division Register
		VADDR_T			gsc_cier_32;	// COS Interrupt Enable Register
		VADDR_T			gsc_cosr_32;	// Change of State Register
		VADDR_T			gsc_cpr_32;		// COS Polarity Register
		VADDR_T			gsc_odr_8;		// Output Data Register
		VADDR_T			gsc_rdr_32;		// Receive Data Register
		VADDR_T			gsc_recr_16;	// Receive Event Counter Register
	} vaddr;
};



// prototypes *****************************************************************

int		dev_irq_create(dev_data_t* dev);
void	dev_irq_destroy(dev_data_t* dev);

int		initialize_ioctl(dev_data_t* dev, void* arg);



#endif
