// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/OPTO32/OPTO32_Linux_7.x.x.x_GSC_DN/driver/opto32.h $
// $Rev: 51654 $
// $Date: 2022-10-07 14:03:38 -0500 (Fri, 07 Oct 2022) $

// OPTO32: Device Driver: header file

#ifndef	__OPTO32_H__
#define	__OPTO32_H__

#include "gsc_common.h"
#include "gsc_pci9056.h"
#include "gsc_pci9060es.h"



// macros *********************************************************************

#define	OPTO32_BASE_NAME					"opto32"
#define	OPTO32_BASE_NAME_LC					OPTO32_BASE_NAME
#define	OPTO32_BASE_NAME_UC					"OPTO32"

// IOCTL command codes
#define	OPTO32_IOCTL_REG_READ				OS_IOCTL_RW( 0, 12, gsc_reg_t)
#define	OPTO32_IOCTL_REG_WRITE				OS_IOCTL_W ( 1, 12, gsc_reg_t)
#define	OPTO32_IOCTL_REG_MOD				OS_IOCTL_W ( 2, 12, gsc_reg_t)
#define	OPTO32_IOCTL_QUERY					OS_IOCTL_RW( 3,  4, s32)
#define	OPTO32_IOCTL_INITIALIZE				OS_IOCTL   ( 4)
#define	OPTO32_IOCTL_CLOCK_DIVIDER			OS_IOCTL_RW( 5,  4, s32)
#define	OPTO32_IOCTL_COS_POLARITY			OS_IOCTL_RW( 6,  4, s32)
#define	OPTO32_IOCTL_COS_STATE				OS_IOCTL_RW( 7,  4, s32)
#define	OPTO32_IOCTL_DEBOUNCE_MS			OS_IOCTL_RW( 8,  4, s32)
#define	OPTO32_IOCTL_DEBOUNCE_US			OS_IOCTL_RW( 9,  4, s32)
#define	OPTO32_IOCTL_IRQ_ENABLE				OS_IOCTL_RW(10,  4, s32)
#define	OPTO32_IOCTL_LED					OS_IOCTL_RW(11,  4, s32)
#define	OPTO32_IOCTL_RX_DATA				OS_IOCTL_R (12,  4, s32)
#define	OPTO32_IOCTL_RX_EVENT_COUNTER		OS_IOCTL_RW(13,  4, s32)
#define	OPTO32_IOCTL_TX_DATA				OS_IOCTL_RW(14,  4, s32)
#define	OPTO32_IOCTL_WAIT_EVENT				OS_IOCTL_RW(15, 28, gsc_wait_t)
#define	OPTO32_IOCTL_WAIT_CANCEL			OS_IOCTL_RW(16, 28, gsc_wait_t)
#define	OPTO32_IOCTL_WAIT_STATUS			OS_IOCTL_RW(17, 28, gsc_wait_t)

//*****************************************************************************
// OPTO32_IOCTL_REG_READ
// OPTO32_IOCTL_REG_WRITE
// OPTO32_IOCTL_REG_MOD
//
#define	OPTO32_REG_ENCODE(s,o)				GSC_REG_ENCODE(GSC_REG_TYPE_BAR2,(s),(o))
// Parameter:	gsc_reg_t*
#define	OPTO32_GSC_BCSR						OPTO32_REG_ENCODE(1,0x00)// Board Control/Status Register
#define	OPTO32_GSC_RDR						OPTO32_REG_ENCODE(4,0x04)// Receive Data Register
#define	OPTO32_GSC_COSR						OPTO32_REG_ENCODE(4,0x08)// Change of State Register
#define	OPTO32_GSC_RECR						OPTO32_REG_ENCODE(2,0x0C)// Receive Event Counter Register
#define	OPTO32_GSC_CIER						OPTO32_REG_ENCODE(4,0x10)// COS Interrupt Enable Register
#define	OPTO32_GSC_CPR						OPTO32_REG_ENCODE(4,0x14)// COS Polarity Register
#define	OPTO32_GSC_CDR						OPTO32_REG_ENCODE(4,0x18)// Clock Division Register
#define	OPTO32_GSC_ODR						OPTO32_REG_ENCODE(1,0x1C)// Output Data Register

//*****************************************************************************
// OPTO32_IOCTL_QUERY
//
//	Parameter:	s32
//		Pass in a value from the list below.
//		The value returned is the answer to the query.

typedef enum
{
	OPTO32_QUERY_COUNT,			//				How many query options are supported?
	OPTO32_QUERY_DEVICE_TYPE,	// PCI ID regs	Value from gsc_dev_type_t

	OPTO32_IOCTL_QUERY_LAST

} opto32_query_t;

#define	OPTO32_IOCTL_QUERY_ERROR			(-1)

//*****************************************************************************
// OPTO32_IOCTL_INITIALIZE					manual operation
//
//	Parameter:	None

//*****************************************************************************
// OPTO32_IOCTL_CLOCK_DIVIDER				CDR D0-D23
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 0xFFFFFF, or
//		-1 to read the current setting.

//*****************************************************************************
// OPTO32_IOCTL_COS_POLARITY				CPR D0-D23
//
//	Parameter:	s32*
//		Pass in any bit combination from 0x000000 to 0xFFFFFF, or
//		-1 to read the current setting.

//*****************************************************************************
// OPTO32_IOCTL_COS_STATE					COS D0-D23
//
//	Parameter:	s32*
//		Pass in any bit combination from 0x000000 to 0xFFFFFF, or
//		-1 to read the current setting.

//*****************************************************************************
// OPTO32_IOCTL_DEBOUNCE_MS					CDR D0-D23
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 2,000, or
//		-1 to read the current setting.
#define	OPTO32_DEBOUNCE_MS_MAX				2000

//*****************************************************************************
// OPTO32_IOCTL_DEBOUNCE_US					CDR D0-D23
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 2,000,000, or
//		-1 to read the current setting.
#define	OPTO32_DEBOUNCE_US_MAX				2000000L

//*****************************************************************************
// OPTO32_IOCTL_IRQ_ENABLE					CIER D0-D23, BCR/BSR D6
//
//	Parameter:	s32*
//		Pass in any combination of the below options, or
//		-1 to read the current setting.
#define	OPTO32_IRQ_COS_00					0x00000001
#define	OPTO32_IRQ_COS_01					0x00000002
#define	OPTO32_IRQ_COS_02					0x00000004
#define	OPTO32_IRQ_COS_03					0x00000008
#define	OPTO32_IRQ_COS_04					0x00000010
#define	OPTO32_IRQ_COS_05					0x00000020
#define	OPTO32_IRQ_COS_06					0x00000040
#define	OPTO32_IRQ_COS_07					0x00000080
#define	OPTO32_IRQ_COS_08					0x00000100
#define	OPTO32_IRQ_COS_09					0x00000200
#define	OPTO32_IRQ_COS_10					0x00000400
#define	OPTO32_IRQ_COS_11					0x00000800
#define	OPTO32_IRQ_COS_12					0x00001000
#define	OPTO32_IRQ_COS_13					0x00002000
#define	OPTO32_IRQ_COS_14					0x00004000
#define	OPTO32_IRQ_COS_15					0x00008000
#define	OPTO32_IRQ_COS_16					0x00010000
#define	OPTO32_IRQ_COS_17					0x00020000
#define	OPTO32_IRQ_COS_18					0x00040000
#define	OPTO32_IRQ_COS_19					0x00080000
#define	OPTO32_IRQ_COS_20					0x00100000
#define	OPTO32_IRQ_COS_21					0x00200000
#define	OPTO32_IRQ_COS_22					0x00400000
#define	OPTO32_IRQ_COS_23					0x00800000
#define	OPTO32_IRQ_EVENT_COUNT				0x01000000

//*****************************************************************************
// OPTO32_IOCTL_LED							BCR/BSR D7
//
//	Parameter:	s32*
//		Pass in any of the below options, or
//		-1 to read the current setting.
#define	OPTO32_LED_OFF						1
#define	OPTO32_LED_ON						0

//*****************************************************************************
// OPTO32_IOCTL_RX_DATA						RDR D0-D23
//
//	Parameter:	s32*
//		The returned value is from 0x000000 to 0xFFFFFF.

//*****************************************************************************
// OPTO32_IOCTL_RX_EVENT_COUNTER			RECR D0-D15
//
//	Parameter:	s32*
//		Pass in any value in the range from 0 to 0xFFFF, or
//		-1 to read the current setting.

//*****************************************************************************
// OPTO32_IOCTL_TX_DATA						ODR D0-D7
//
//	Parameter:	s32*
//		Pass in any value in the range from 0x00 to 0xFF, or
//		-1 to read the current setting.

//*****************************************************************************
// OPTO32_IOCTL_WAIT_EVENT		all fields must be valid
// OPTO32_IOCTL_WAIT_CANCEL		fields need not be valid
// OPTO32_IOCTL_WAIT_STATUS		fields need not be valid
//
//	Parameter:	gsc_wait_t*
// gsc_wait_t.flags - see gsc_common.h
// gsc_wait_t.main - see gsc_common.h
// gsc_wait_t.gsc
#define	OPTO32_WAIT_GSC_COS_00				OPTO32_IRQ_COS_00
#define	OPTO32_WAIT_GSC_COS_01				OPTO32_IRQ_COS_01
#define	OPTO32_WAIT_GSC_COS_02				OPTO32_IRQ_COS_02
#define	OPTO32_WAIT_GSC_COS_03				OPTO32_IRQ_COS_03
#define	OPTO32_WAIT_GSC_COS_04				OPTO32_IRQ_COS_04
#define	OPTO32_WAIT_GSC_COS_05				OPTO32_IRQ_COS_05
#define	OPTO32_WAIT_GSC_COS_06				OPTO32_IRQ_COS_06
#define	OPTO32_WAIT_GSC_COS_07				OPTO32_IRQ_COS_07
#define	OPTO32_WAIT_GSC_COS_08				OPTO32_IRQ_COS_08
#define	OPTO32_WAIT_GSC_COS_09				OPTO32_IRQ_COS_09
#define	OPTO32_WAIT_GSC_COS_10				OPTO32_IRQ_COS_10
#define	OPTO32_WAIT_GSC_COS_11				OPTO32_IRQ_COS_11
#define	OPTO32_WAIT_GSC_COS_12				OPTO32_IRQ_COS_12
#define	OPTO32_WAIT_GSC_COS_13				OPTO32_IRQ_COS_13
#define	OPTO32_WAIT_GSC_COS_14				OPTO32_IRQ_COS_14
#define	OPTO32_WAIT_GSC_COS_15				OPTO32_IRQ_COS_15
#define	OPTO32_WAIT_GSC_COS_16				OPTO32_IRQ_COS_16
#define	OPTO32_WAIT_GSC_COS_17				OPTO32_IRQ_COS_17
#define	OPTO32_WAIT_GSC_COS_18				OPTO32_IRQ_COS_18
#define	OPTO32_WAIT_GSC_COS_19				OPTO32_IRQ_COS_19
#define	OPTO32_WAIT_GSC_COS_20				OPTO32_IRQ_COS_20
#define	OPTO32_WAIT_GSC_COS_21				OPTO32_IRQ_COS_21
#define	OPTO32_WAIT_GSC_COS_22				OPTO32_IRQ_COS_22
#define	OPTO32_WAIT_GSC_COS_23				OPTO32_IRQ_COS_23
#define	OPTO32_WAIT_GSC_EVENT_COUNT			OPTO32_IRQ_EVENT_COUNT
#define	OPTO32_WAIT_GSC_ALL					0x1FFFFFF
// gsc_wait_t.alt flags
#define	OPTO32_WAIT_ALT_ALL					0
// gsc_wait_t.io
#define	OPTO32_WAIT_IO_ALL					0



#endif
