// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/OPTO32/OPTO32_Linux_7.x.x.x_GSC_DN/samples/sbtest/reg_mod.c $
// $Rev: 51211 $
// $Date: 2022-06-30 11:15:26 -0500 (Thu, 30 Jun 2022) $

// OPTO32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	mask;
	} list[]	=
	{
		// reg					mask
		{ OPTO32_GSC_CPR,	0x00FFFFFF	},
		{ OPTO32_GSC_CDR,	0x00FFFFFF	},
		{ OPTO32_GSC_ODR,	0x000000FF	},
	};

	u32			bit;
	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;
	u32			value;

	for (i = 0; (errs == 0) && (i < (int) SIZEOF_ARRAY(list)); i++)
	{

		for (bit = 0x1; bit; bit <<= 1)
		{
			if ((list[i].mask & bit) == 0)
				continue;

			// Read the register value so we can restore it later.
			parm.reg	= list[i].reg;
			ret			= opto32_ioctl(fd, OPTO32_IOCTL_REG_READ, (void*) &parm);
			value		= parm.value;

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, opto32_ioctl() returned %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Set the bit low.
			parm.value	= 0;
			parm.mask	= bit;
			ret			= opto32_ioctl(fd, OPTO32_IOCTL_REG_MOD, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, opto32_ioctl() returned %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Now verify that the bit is low.
			ret	= opto32_ioctl(fd, OPTO32_IOCTL_REG_READ, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, opto32_ioctl() returned %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			if (parm.value & bit)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not low, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= opto32_ioctl(fd, OPTO32_IOCTL_REG_WRITE, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, opto32_ioctl() returned %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Set the bit high.
			parm.value	= bit;
			parm.mask	= bit;
			ret			= opto32_ioctl(fd, OPTO32_IOCTL_REG_MOD, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, opto32_ioctl() returned %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			// Now verify that the bit is high.
			ret	= opto32_ioctl(fd, OPTO32_IOCTL_REG_READ, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, opto32_ioctl() returned %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}

			if ((parm.value & bit) == 0)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, bit not high, mask 0x%lX, got 0x%lX)\n",
						__LINE__,
						i,
						(long) bit,
						(long) parm.value);
				break;
			}

			// Now restore the register to its previous value.
			parm.value	= value;
			ret			= opto32_ioctl(fd, OPTO32_IOCTL_REG_WRITE, (void*) &parm);

			if (ret)
			{
				errs	= 1;
				printf(	"FAIL <---  (%d. i %d, opto32_ioctl() returned %d)\n",
						__LINE__,
						i,
						ret);
				break;
			}
		}
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	reg_mod_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service OPTO32_IOCTL_REG_MOD.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_mod_test(int fd)
{
	int	errs	= 0;

	gsc_label("OPTO32_IOCTL_REG_MOD");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


