// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/OPTO32/OPTO32_Linux_7.x.x.x_GSC_DN/samples/sbtest/reg_write.c $
// $Rev: 51211 $
// $Date: 2022-06-30 11:15:26 -0500 (Thu, 30 Jun 2022) $

// OPTO32: Sample Application: source file

#include "main.h"



//*****************************************************************************
static int _service_test(int fd)
{
	return(0);
}



//*****************************************************************************
static int _function_test(int fd)
{
	static const struct
	{
		u32	reg;
		u32	value;
		u32	mask;
	} list[]	=
	{
		// reg					value		mask
		{ OPTO32_GSC_CPR,	0x0055AA55,	0x00FFFFFF	},
		{ OPTO32_GSC_CPR,	0x00AA55AA,	0x00FFFFFF	},
		{ OPTO32_GSC_CPR,	0x00000000,	0x00FFFFFF	},

		{ OPTO32_GSC_CDR,	0x00AA55AA,	0x00FFFFFF	},
		{ OPTO32_GSC_CDR,	0x0055AA55,	0x00FFFFFF	},
		{ OPTO32_GSC_CDR,	0x00000000,	0x00FFFFFF	},

		{ OPTO32_GSC_ODR,	0x00000055,	0x000000FF	},
		{ OPTO32_GSC_ODR,	0x000000AA,	0x000000FF	},
		{ OPTO32_GSC_ODR,	0x00000000,	0x000000FF	},
	};

	int			errs	= 0;
	int			i;
	gsc_reg_t	parm;
	int			ret;

	for (i = 0; i < (int) SIZEOF_ARRAY(list); i++)
	{
		parm.reg	= list[i].reg;
		parm.value	= list[i].value;
		parm.mask	= 0xBEEFDEAD;
		ret			= opto32_ioctl(fd, OPTO32_IOCTL_REG_WRITE, (void*) &parm);

		if (ret)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, opto32_ioctl() returned %d)\n",
					__LINE__,
					i,
					ret);
			break;
		}

		if (parm.mask != 0xBEEFDEAD)
		{
			errs	= 1;
			printf("FAIL <---  (%d. i %d, mask changed)\n", __LINE__, i);
			break;
		}

		// Now read the register and verify the bits of interest.
		ret	= opto32_ioctl(fd, OPTO32_IOCTL_REG_READ, (void*) &parm);

		if (ret)
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, opto32_ioctl() returned %d)\n",
					__LINE__,
					i,
					ret);
			break;
		}

		if ((list[i].value & list[i].mask) != (parm.value & list[i].mask))
		{
			errs	= 1;
			printf(	"FAIL <---  (%d. i %d, mask 0x%lX, got 0x%lX, expected 0x%lX)\n",
					__LINE__,
					i,
					(long) list[i].mask,
					(long) parm.value,
					(long) list[i].value);
			break;
		}
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	reg_write_test
*
*	Purpose:
*
*		Perform a test of the IOCTL service OPTO32_IOCTL_REG_WRITE.
*
*	Arguments:
*
*		fd		The handle for the device to access.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int reg_write_test(int fd)
{
	int	errs	= 0;

	gsc_label("OPTO32_IOCTL_REG_WRITE");
	errs	+= _service_test(fd);
	errs	+= _function_test(fd);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


