// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/OPTO32/OPTO32_Linux_7.x.x.x_GSC_DN/utils/util_reg.c $
// $Rev: 51204 $
// $Date: 2022-06-30 11:00:14 -0500 (Thu, 30 Jun 2022) $

// OPTO32: Utilities: source file

#include "main.h"



// macros *********************************************************************

#define	_GSC_REG(a)				"GSC " #a, OPTO32_GSC_##a, 0, 0
#define	SHOW_FIELD(b,e,eol,l,n)	gsc_reg_field_show(width+10,27,value,(b),(e),(eol),(l),(n))
#define	SIZEOF_ARRAY(a)			(sizeof((a)) / sizeof((a)[0]))



//*****************************************************************************
static int _bcsr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	disable[]	= { "Disable",	"Enable"	};
	static const char*	idle[]		= { "Idle",		"Active"	};
	static const char*	on[]		= { "On",		"Off"		};

	SHOW_FIELD(7, 7, 1, on,			"LED"						);
	SHOW_FIELD(6, 6, 1, disable,	"Event Overflow IRQ"		);
	SHOW_FIELD(5, 5, 1, NULL,		"Slow Debounce Clock"		);
	SHOW_FIELD(4, 4, 1, idle,		"Master IRQ Status"			);
	SHOW_FIELD(3, 3, 1, idle,		"Event Overflow IRQ Status"	);
	SHOW_FIELD(2, 2, 1, idle,		"COS 16-23 IRQ Status"		);
	SHOW_FIELD(1, 1, 1, idle,		"COS 8-15 IRQ Status"		);
	SHOW_FIELD(0, 0, 1, idle,		"COS 0-7 IRQ Status"		);
	return(0);
}



//*****************************************************************************
static int _rdr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(31, 24, 1, NULL,	"Reserved"	);
	SHOW_FIELD(23,  0, 1, NULL,	"Data"		);
	return(0);
}



//*****************************************************************************
static int _cosr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	idle[]	= { "Idle",	"Active"	};

	SHOW_FIELD(31, 24, 1, NULL,	"Reserved"	);
	SHOW_FIELD(23, 23, 1, idle,	"COS 23"	);
	SHOW_FIELD(22, 22, 1, idle,	"COS 22"	);
	SHOW_FIELD(21, 21, 1, idle,	"COS 21"	);
	SHOW_FIELD(20, 20, 1, idle,	"COS 20"	);
	SHOW_FIELD(19, 19, 1, idle,	"COS 19"	);
	SHOW_FIELD(18, 18, 1, idle,	"COS 18"	);
	SHOW_FIELD(17, 17, 1, idle,	"COS 17"	);
	SHOW_FIELD(16, 16, 1, idle,	"COS 16"	);
	SHOW_FIELD(15, 15, 1, idle,	"COS 15"	);
	SHOW_FIELD(14, 14, 1, idle,	"COS 14"	);
	SHOW_FIELD(13, 13, 1, idle,	"COS 13"	);
	SHOW_FIELD(12, 12, 1, idle,	"COS 12"	);
	SHOW_FIELD(11, 11, 1, idle,	"COS 11"	);
	SHOW_FIELD(10, 10, 1, idle,	"COS 10"	);
	SHOW_FIELD( 9,  9, 1, idle,	"COS 9"		);
	SHOW_FIELD( 8,  8, 1, idle,	"COS 8"		);
	SHOW_FIELD( 7,  7, 1, idle,	"COS 7"		);
	SHOW_FIELD( 6,  6, 1, idle,	"COS 6"		);
	SHOW_FIELD( 5,  5, 1, idle,	"COS 5"		);
	SHOW_FIELD( 4,  4, 1, idle,	"COS 4"		);
	SHOW_FIELD( 3,  3, 1, idle,	"COS 3"		);
	SHOW_FIELD( 2,  2, 1, idle,	"COS 2"		);
	SHOW_FIELD( 1,  1, 1, idle,	"COS 1"		);
	SHOW_FIELD( 0,  0, 1, idle,	"COS 0"		);
	return(0);
}



//*****************************************************************************
static int _recr_detail(int fd, int supported, u32 value, int width)
{
	SHOW_FIELD(15, 0, 1, NULL,	"Count"	);

	return(0);
}



//*****************************************************************************
static int _cier_detail(int fd, int supported, u32 value, int width)
{
	static const char*	disabled[]	= { "Disabled",	"Enabled"	};

	SHOW_FIELD(31, 24, 1, NULL,		"Reserved"	);
	SHOW_FIELD(23, 23, 1, disabled,	"COS 23"	);
	SHOW_FIELD(22, 22, 1, disabled,	"COS 22"	);
	SHOW_FIELD(21, 21, 1, disabled,	"COS 21"	);
	SHOW_FIELD(20, 20, 1, disabled,	"COS 20"	);
	SHOW_FIELD(19, 19, 1, disabled,	"COS 19"	);
	SHOW_FIELD(18, 18, 1, disabled,	"COS 18"	);
	SHOW_FIELD(17, 17, 1, disabled,	"COS 17"	);
	SHOW_FIELD(16, 16, 1, disabled,	"COS 16"	);
	SHOW_FIELD(15, 15, 1, disabled,	"COS 15"	);
	SHOW_FIELD(14, 14, 1, disabled,	"COS 14"	);
	SHOW_FIELD(13, 13, 1, disabled,	"COS 13"	);
	SHOW_FIELD(12, 12, 1, disabled,	"COS 12"	);
	SHOW_FIELD(11, 11, 1, disabled,	"COS 11"	);
	SHOW_FIELD(10, 10, 1, disabled,	"COS 10"	);
	SHOW_FIELD( 9,  9, 1, disabled,	"COS 9"		);
	SHOW_FIELD( 8,  8, 1, disabled,	"COS 8"		);
	SHOW_FIELD( 7,  7, 1, disabled,	"COS 7"		);
	SHOW_FIELD( 6,  6, 1, disabled,	"COS 6"		);
	SHOW_FIELD( 5,  5, 1, disabled,	"COS 5"		);
	SHOW_FIELD( 4,  4, 1, disabled,	"COS 4"		);
	SHOW_FIELD( 3,  3, 1, disabled,	"COS 3"		);
	SHOW_FIELD( 2,  2, 1, disabled,	"COS 2"		);
	SHOW_FIELD( 1,  1, 1, disabled,	"COS 1"		);
	SHOW_FIELD( 0,  0, 1, disabled,	"COS 0"		);
	return(0);
}



//*****************************************************************************
static int _cpr_detail(int fd, int supported, u32 value, int width)
{
	static const char*	h2l[]	= { "High-to-Low",	"Low-to-High"	};

	SHOW_FIELD(31, 24, 1, NULL,	"Reserved"	);
	SHOW_FIELD(23, 23, 1, h2l,	"COS 23"	);
	SHOW_FIELD(22, 22, 1, h2l,	"COS 22"	);
	SHOW_FIELD(21, 21, 1, h2l,	"COS 21"	);
	SHOW_FIELD(20, 20, 1, h2l,	"COS 20"	);
	SHOW_FIELD(19, 19, 1, h2l,	"COS 19"	);
	SHOW_FIELD(18, 18, 1, h2l,	"COS 18"	);
	SHOW_FIELD(17, 17, 1, h2l,	"COS 17"	);
	SHOW_FIELD(16, 16, 1, h2l,	"COS 16"	);
	SHOW_FIELD(15, 15, 1, h2l,	"COS 15"	);
	SHOW_FIELD(14, 14, 1, h2l,	"COS 14"	);
	SHOW_FIELD(13, 13, 1, h2l,	"COS 13"	);
	SHOW_FIELD(12, 12, 1, h2l,	"COS 12"	);
	SHOW_FIELD(11, 11, 1, h2l,	"COS 11"	);
	SHOW_FIELD(10, 10, 1, h2l,	"COS 10"	);
	SHOW_FIELD( 9,  9, 1, h2l,	"COS 9"		);
	SHOW_FIELD( 8,  8, 1, h2l,	"COS 8"		);
	SHOW_FIELD( 7,  7, 1, h2l,	"COS 7"		);
	SHOW_FIELD( 6,  6, 1, h2l,	"COS 6"		);
	SHOW_FIELD( 5,  5, 1, h2l,	"COS 5"		);
	SHOW_FIELD( 4,  4, 1, h2l,	"COS 4"		);
	SHOW_FIELD( 3,  3, 1, h2l,	"COS 3"		);
	SHOW_FIELD( 2,  2, 1, h2l,	"COS 2"		);
	SHOW_FIELD( 1,  1, 1, h2l,	"COS 1"		);
	SHOW_FIELD( 0,  0, 1, h2l,	"COS 0"		);
	return(0);
}



//*****************************************************************************
static int _cdr_detail(int fd, int supported, u32 value, int width)
{
	u32	div;

	SHOW_FIELD(31, 24, 1, NULL,	"Reserved"	);

	SHOW_FIELD(23, 0, 0, NULL,	"Divider"	);
	div	= value & 0xFFFFFF;
	printf(	"%ld (0x%lX, /%ld)\n",
			(long) div,
			(long) div,
			(long) ((div <= 1) ? 2 : div));

	return(0);
}



// variables ******************************************************************

static gsc_reg_def_t	_gsc[]	=
{
	{ _GSC_REG(BCSR),	0,	_bcsr_detail,	"Board Control/Status Register"		},
	{ _GSC_REG(RDR),	0,	_rdr_detail,	"Receive Data Register"				},
	{ _GSC_REG(COSR),	0,	_cosr_detail,	"Change of State Register"			},
	{ _GSC_REG(RECR),	0,	_recr_detail,	"Receive Event Counter Register"	},
	{ _GSC_REG(CIER),	0,	_cier_detail,	"COS Interrupt EnableRegister"		},
	{ _GSC_REG(CPR),	0,	_cpr_detail,	"COS Polarity Register"				},
	{ _GSC_REG(CDR),	0,	_cdr_detail,	"Clock Divider Register"			},
	{ _GSC_REG(ODR),	0,	NULL,			"Output Data Register"				},
	{ NULL, 0, 0, 0, 0, 0,	NULL	}
};



//*****************************************************************************
static const gsc_reg_def_t* _find_reg(u32 reg, const gsc_reg_def_t* list)
{
	const gsc_reg_def_t*	def	= NULL;
	int						i;

	for (i = 0; list[i].name; i++)
	{
		if (reg == list[i].reg)
		{
			def	= &list[i];
			break;
		}
	}

	return(def);
}



/******************************************************************************
*
*	Function:	opto32_reg_get_def_id
*
*	Purpose:
*
*		Retrieve the register definition structure given the register id.
*
*	Arguments:
*
*		reg		The id of the register to access.
*
*	Returned:
*
*		NULL	The register id wasn't found.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* opto32_reg_get_def_id(u32 reg)
{
	const gsc_reg_def_t*	def;

	def	= _find_reg(reg, _gsc);
	return(def);
}



/******************************************************************************
*
*	Function:	opto32_reg_get_def_index
*
*	Purpose:
*
*		Retrieve the register definition structure based on an index.
*
*	Arguments:
*
*		index	The index of the register to access.
*
*	Returned:
*
*		NULL	The index doesn't correspond to a known register.
*		else	A pointer to the register definition.
*
******************************************************************************/

const gsc_reg_def_t* opto32_reg_get_def_index(int index)
{
	const gsc_reg_def_t*	def;

	if (index < 0)
		def	= NULL;
	else if (index >= (int) (SIZEOF_ARRAY(_gsc) - 1))
		def	= NULL;
	else
		def	= &_gsc[index];

	return(def);
}



/******************************************************************************
*
*	Function:	opto32_reg_get_desc
*
*	Purpose:
*
*		Retrieve the description of the specified register.
*
*	Arguments:
*
*		reg		The register whose description is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* opto32_reg_get_desc(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				desc;

	def	= _find_reg(reg, _gsc);

	if (def)
		desc	= def->desc;
	else
		desc	= "UNKNOWN";

	return(desc);
}



/******************************************************************************
*
*	Function:	opto32_reg_get_name
*
*	Purpose:
*
*		Retrieve the name of the specified register.
*
*	Arguments:
*
*		reg		The register whose name is desired.
*
*	Returned:
*
*		!NULL	The register's name.
*
******************************************************************************/

const char* opto32_reg_get_name(u32 reg)
{
	const gsc_reg_def_t*	def;
	const char*				name;

	def	= _find_reg(reg, _gsc);

	if (def)
		name	= def->name;
	else
		name	= "UNKNOWN";

	return(name);
}



/******************************************************************************
*
*	Function:	opto32_reg_list
*
*	Purpose:
*
*		List the GSC registers and their values.
*
*	Arguments:
*
*		fd		The handle to access the device.
*
*		detail	List the register details?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int opto32_reg_list(int fd, int detail)
{
	int	errs;

	errs	= gsc_reg_list(fd, _gsc, detail, opto32_reg_read);
	return(errs);
}



